#
# Interface type

getsym $coff _iftype
PutLong $DEV $_iftype_addr $_iftype_pg $iftype

#
# Transfer controller base address to target-side code
#
getsym $coff _ctrlr_base
PutLong $DEV $_ctrlr_base_addr $_ctrlr_base_pg $ctrlr_base

#
# Initialization for I2C controller
#

# pinmux for L-138 or 674x

  if { [string equal $cpu_type "TMS320C674x"] } {
  
    set PINMUX4 [expr 0x01C14000 + 0x130]

    if { $ifnum == 1 } {

      set rval [ ReadLong $DEV $PINMUX4 0 ]
      set rval [ expr $rval & ~0x00FF0000 ]
      set rval [ expr $rval |  0x00440000 ]
      PutLong $DEV $PINMUX4 0 $rval

      puts "I2C1 pinmux configured"

    }

    if { $ifnum == 0 } {

      set rval [ ReadLong $DEV $PINMUX4 0 ]
      set rval [ expr $rval & ~0x0000FF00 ]
      set rval [ expr $rval |  0x00002200 ]
      PutLong $DEV $PINMUX4 0 $rval

      puts "I2C0 pinmux configured"

    }

  }

#
# Initialize I2C controller itself
#
if { [string equal $cpu_type "TMS320C5500"] ||
     [string equal $cpu_type "TMS470R1B"  ] ||
     [string equal $cpu_type "TMS320C6720"] ||
     [string equal $cpu_type "TMS320C2800"] ||
     [string equal $cpu_type "TMS320C674x"] } {

  if { [string equal $cpu_type "TMS470R1B"] } {
    set I2COAR  0x00
    set I2CIMR  0x04
    set I2CSTR  0x08
    set I2CCLKL 0x0C
    set I2CCLKH 0x10
    set I2CCNT  0x14
    set I2CDRR  0x18
    set I2CSAR  0x1C
    set I2CDXR  0x20
    set I2CMDR  0x24
    set I2CIVR  0x28
    set I2CEMR  0x2C
    set I2CPSC  0x30
    set I2CDIR  0x34
    set I2CDOUT 0x38
    set I2CDIN  0x3C
  } elseif { [string equal $cpu_type "TMS320C6720"] ||
             [string equal $cpu_type "TMS320C674x"] } {
    set I2COAR  0x00
    set I2CIMR  0x04
    set I2CSTR  0x08
    set I2CCLKL 0x0C
    set I2CCLKH 0x10
    set I2CCNT  0x14
    set I2CDRR  0x18
    set I2CSAR  0x1C
    set I2CDXR  0x20
    set I2CMDR  0x24
    set I2CIVR  0x28
    set I2CEMR  0x2C
    set I2CPSC  0x30
    set I2CPFUNC 0x48
    set I2CPDIR  0x4C
    set I2CPDIN  0x50
    set I2CPDOUT 0x54
    set I2CPDSET 0x58
    set I2CPDCLR 0x5C
  } else {
    set I2COAR  0x00
    set I2CIER  0x01
    set I2CSTR  0x02
    set I2CCLKL 0x03
    set I2CCLKH 0x04
    set I2CCNT  0x05
    set I2CDRR  0x06
    set I2CSAR  0x07
    set I2CDXR  0x08
    set I2CMDR  0x09
    set I2CIVR  0x0A
    set I2CGPIO 0x0B
    set I2CPSC  0x0C
  }

  set mempage 1

  if { [string equal $cpu_type "TMS320C5500"] } {

    set mempage 2

  } elseif { [string equal $cpu_type "TMS470R1B"] } {

    set mempage 0
    # enable I2C pins on TMS470
    puts "Enabling I2C pins." 
    PutShort $DEV [expr $ctrlr_base + $I2CDIR] $mempage 0x000C

  } elseif { [string equal $cpu_type "TMS320C6720"] ||
             [string equal $cpu_type "TMS320C674x"] } {

    set mempage 0
    # enable I2C pins on C672x
    puts "Enabling I2C pins." 
    PutLong $DEV [expr $ctrlr_base + $I2CPFUNC] $mempage 0

  }

  puts "Initializing I2C controller..." 

  #
  # calculate divisor for 100 KHz
  set divisor [expr (($periph_clock / 100000) - 14) / 2 ]

  puts "Calculated I2CCLKL/H values = $divisor"

  # Set "FREE" bit and wait some time
  set rval [ReadShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage]
  set rval [expr $rval | 0x4000]
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage $rval
  after 200

  # Reset; clear 0x20 in I2CMDR
  set rval [ReadShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage]
  set rval [expr $rval & 0xFFDF]
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage $rval
  after 100

  # I2CCNT
  PutShort $DEV [expr $ctrlr_base + $I2CCNT] $mempage 0
  # I2CPSC
  PutShort $DEV [expr $ctrlr_base + $I2CPSC] $mempage 0
  # I2CCLKL
  PutShort $DEV [expr $ctrlr_base + $I2CCLKL] $mempage $divisor
  # I2CCLKH
  PutShort $DEV [expr $ctrlr_base + $I2CCLKH] $mempage $divisor
  # Reset status; I2CSTR <= FFFF
  PutShort $DEV [expr $ctrlr_base + $I2CSTR] $mempage 0xFFFF
  after 10

  #I2CMDR out of RESET
  set rval [ReadShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage]
  set rval [expr $rval | 0x20]
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage $rval
  after 10

  #########################################
  # Send START + STOP to reset I2C slaves #
  #########################################

  # Initiate START condition
  # FREE + STT + MST + TRX + RM + IRS + FDF
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage 0x66A8
  # wait for START generated
  for { set ii 0 } { $ii < 100 } { incr ii } {
    set rval [ReadShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage]
    if { ! ($rval & 0x2000) } {
      break
    }
    after 10
  }

  if { $rval & 0x2000 } {
    ferror $DEV "Timeout 1 initializing I2C bus" 1
  }

  # Initiate STOP condition
  # FREE + STP + MST + TRX + RM + IRS + FDF
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage 0x4EA8
  # wait for START generated
  for { set ii 0 } { $ii < 100 } { incr ii } {
    set rval [ReadShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage]
    if { ! ($rval & 0x0800) } {
      break
    }
    after 10
  }

  if { $rval & 0x0800 } {
    ferror $DEV "Timeout 2 initializing I2C bus" 1
  }

  # FREE + MST + IRS
  PutShort $DEV [expr $ctrlr_base + $I2CMDR] $mempage 0x4420
  # Clear all status bits
  PutShort $DEV [expr $ctrlr_base + $I2CSTR] $mempage 0xFFFF

}
